//
// Copyright (c) 2002-2010 The ANGLE Project Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
//

#ifndef _RESOURCE_LIMITS_INCLUDED_
#define _RESOURCE_LIMITS_INCLUDED_

struct TBuiltInResource
{
	int maxVertexAttribs;
	int maxVertexUniformVectors;
	int maxVaryingVectors;
	int maxVertexTextureImageUnits;
	int maxCombinedTextureImageUnits;
	int maxTextureImageUnits;
	int maxFragmentUniformVectors;
	int maxDrawBuffers;
};
#endif // _RESOURCE_LIMITS_INCLUDED_
