/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM /builds/moz2_slave/mozilla-central-linux-xulrunner/build/dom/indexedDB/nsIIDBKeyRange.idl
 */

#ifndef __gen_nsIIDBKeyRange_h__
#define __gen_nsIIDBKeyRange_h__


#ifndef __gen_nsISupports_h__
#include "nsISupports.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIVariant; /* forward declaration */


/* starting interface:    nsIIDBKeyRange */
#define NS_IIDBKEYRANGE_IID_STR "8e6b05f4-2358-420b-a6fd-03416543ed18"

#define NS_IIDBKEYRANGE_IID \
  {0x8e6b05f4, 0x2358, 0x420b, \
    { 0xa6, 0xfd, 0x03, 0x41, 0x65, 0x43, 0xed, 0x18 }}

/**
 * IDBKeyRange interface.  See
 * http://dev.w3.org/2006/webapi/WebSimpleDB/#idl-def-IDBKeyRange for more
 * information.
 */
class NS_NO_VTABLE NS_SCRIPTABLE nsIIDBKeyRange : public nsISupports {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IIDBKEYRANGE_IID)

  enum { LEFT_OPEN = 2U };

  enum { RIGHT_OPEN = 4U };

  enum { LEFT_BOUND = 8U };

  enum { RIGHT_BOUND = 16U };

  /* readonly attribute unsigned short flags; */
  NS_SCRIPTABLE NS_IMETHOD GetFlags(PRUint16 *aFlags) = 0;

  /* readonly attribute nsIVariant left; */
  NS_SCRIPTABLE NS_IMETHOD GetLeft(nsIVariant **aLeft) = 0;

  /* readonly attribute nsIVariant right; */
  NS_SCRIPTABLE NS_IMETHOD GetRight(nsIVariant **aRight) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIIDBKeyRange, NS_IIDBKEYRANGE_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIIDBKEYRANGE \
  NS_SCRIPTABLE NS_IMETHOD GetFlags(PRUint16 *aFlags); \
  NS_SCRIPTABLE NS_IMETHOD GetLeft(nsIVariant **aLeft); \
  NS_SCRIPTABLE NS_IMETHOD GetRight(nsIVariant **aRight); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIIDBKEYRANGE(_to) \
  NS_SCRIPTABLE NS_IMETHOD GetFlags(PRUint16 *aFlags) { return _to GetFlags(aFlags); } \
  NS_SCRIPTABLE NS_IMETHOD GetLeft(nsIVariant **aLeft) { return _to GetLeft(aLeft); } \
  NS_SCRIPTABLE NS_IMETHOD GetRight(nsIVariant **aRight) { return _to GetRight(aRight); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIIDBKEYRANGE(_to) \
  NS_SCRIPTABLE NS_IMETHOD GetFlags(PRUint16 *aFlags) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetFlags(aFlags); } \
  NS_SCRIPTABLE NS_IMETHOD GetLeft(nsIVariant **aLeft) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetLeft(aLeft); } \
  NS_SCRIPTABLE NS_IMETHOD GetRight(nsIVariant **aRight) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetRight(aRight); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsIDBKeyRange : public nsIIDBKeyRange
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIIDBKEYRANGE

  nsIDBKeyRange();

private:
  ~nsIDBKeyRange();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsIDBKeyRange, nsIIDBKeyRange)

nsIDBKeyRange::nsIDBKeyRange()
{
  /* member initializers and constructor code */
}

nsIDBKeyRange::~nsIDBKeyRange()
{
  /* destructor code */
}

/* readonly attribute unsigned short flags; */
NS_IMETHODIMP nsIDBKeyRange::GetFlags(PRUint16 *aFlags)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute nsIVariant left; */
NS_IMETHODIMP nsIDBKeyRange::GetLeft(nsIVariant **aLeft)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* readonly attribute nsIVariant right; */
NS_IMETHODIMP nsIDBKeyRange::GetRight(nsIVariant **aRight)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIIDBKeyRange_h__ */
